package com.example.f1hub.ui.view

import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.TextView
import android.widget.Toast

import androidx.activity.enableEdgeToEdge
import androidx.appcompat.app.AppCompatActivity
import androidx.lifecycle.lifecycleScope

import androidx.navigation.NavController
import androidx.navigation.fragment.NavHostFragment
import androidx.navigation.ui.AppBarConfiguration
import androidx.navigation.ui.navigateUp
import androidx.navigation.ui.setupActionBarWithNavController
import androidx.navigation.ui.setupWithNavController

import com.bumptech.glide.Glide
import com.bumptech.glide.load.resource.bitmap.CircleCrop

import com.example.f1hub.R
import com.example.f1hub.data.dto.`in`.users.UserResponse
import com.example.f1hub.databinding.ActivityMainBinding
import com.example.f1hub.domain.usecase.users.FindUserByIdUseCase
import com.example.f1hub.ui.view.login.Login

import com.google.firebase.auth.FirebaseAuth
import dagger.hilt.android.AndroidEntryPoint

import kotlinx.coroutines.launch

import javax.inject.Inject

@AndroidEntryPoint
class MainActivity : AppCompatActivity() {
    @Inject lateinit var findUserByIdUseCase: FindUserByIdUseCase
    @Inject lateinit var sharedPreferences: SharedPreferences

    lateinit var navController: NavController
    private lateinit var appBarConfiguration: AppBarConfiguration
    lateinit var binding: ActivityMainBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityMainBinding.inflate(layoutInflater)
        enableEdgeToEdge() // Activa el modo de pantalla completa para mostrar la app a pantalla completa
        setContentView(binding.root) // Establece el contenido de la actividad

        val decorView = window.decorView
        val uiOptions = View.SYSTEM_UI_FLAG_FULLSCREEN or
                View.SYSTEM_UI_FLAG_HIDE_NAVIGATION or
                View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY
        decorView.systemUiVisibility = uiOptions


        //configuramos el nombre de usuario y correo
        loadUserInfo()

        val toolbar = binding.appBarLayoutDrawer.toolbar
        val navHost = supportFragmentManager.findFragmentById(R.id.fragmentContainerView) as NavHostFragment
        navController = navHost.navController
        val navView = binding.myNavView
        appBarConfiguration = AppBarConfiguration(
            setOf(R.id.homeFragment, R.id.settingsFragment, R.id.itemsFragment), // Destinos principales
            binding.main
        )

        //insertamos la toolbar
        setSupportActionBar(toolbar)
        supportActionBar!!.setDisplayShowTitleEnabled(false)

        setupActionBarWithNavController(navController, appBarConfiguration)


        navView.setupWithNavController(navController)

    }

    override fun onSupportNavigateUp(): Boolean{
        return navController.navigateUp(appBarConfiguration) || super.onSupportNavigateUp()
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        menuInflater.inflate(R.menu.toolbar_op, menu)
        return true
    }


    //Navegación del menú de opciones.
    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            R.id.homeFragment -> {
                navController.navigate(R.id.homeFragment)
                true
            }
            R.id.settingsFragment -> {
                navController.navigate(R.id.settingsFragment,)
                true
            }
            R.id.itemsFragment -> {
                navController.navigate(R.id.itemsFragment)
                true
            }
            R.id.logout -> {
                logout()
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }
    private fun logout() {
        sharedPreferences.edit()
            .remove("auth_token")
            .remove("user_id")
            .apply()
        val intent = Intent(this, Login::class.java)
        startActivity(intent)
        finish()
    }

        private fun loadUserInfo() {
        val token = sharedPreferences.getString("auth_token", "") ?: ""
        val userId = sharedPreferences.getInt("user_id", -1)

        if (token.isNotEmpty() && userId > 0) {
            lifecycleScope.launch {
                try {
                    val user = findUserByIdUseCase(token, userId)
                    if (user != null) {
                        updateUI(user)
                    } else {
                        showToast("No se pudo cargar la información del usuario")
                    }
                } catch (e: Exception) {
                    showToast("Error al obtener usuario: ${e.message}")
                }
            }
        }
    }

    private fun updateUI(user: UserResponse) {
        val headerView = binding.myNavView.getHeaderView(0)
        headerView.findViewById<TextView>(R.id.txt_name).text = user.username
        headerView.findViewById<TextView>(R.id.txt_email).text = user.email

        Glide.with(this)
            .load(user.profilePhoto)
            .placeholder(R.drawable.loading_bar)
            .error(R.drawable.not_found)
            .override(300, 300)
            .transform(CircleCrop())
            .into(headerView.findViewById(R.id.image_perfil))
    }

    private fun showToast(message: String) {
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show()
    }

}