package com.example.f1hub.ui.view.fragment

import android.content.ContentResolver
import android.content.SharedPreferences
import android.net.Uri
import android.os.Bundle
import android.util.Base64
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.lifecycle.lifecycleScope
import com.bumptech.glide.Glide
import com.bumptech.glide.load.resource.bitmap.CircleCrop
import com.example.f1hub.R
import com.example.f1hub.data.dto.`in`.users.UserResponse
import com.example.f1hub.data.dto.out.users.UserAllAttributesDto
import com.example.f1hub.databinding.FragmentSettingsBinding
import com.example.f1hub.domain.usecase.users.FindUserByIdUseCase
import com.example.f1hub.domain.usecase.users.UpdateUserUseCase
import com.example.f1hub.ui.view.MainActivity
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.launch
import java.io.InputStream
import javax.inject.Inject

@AndroidEntryPoint
class SettingsFragment : Fragment() {

    private lateinit var binding: FragmentSettingsBinding
    private var selectedImageUri: Uri? = null // URI de la imagen seleccionada

    @Inject lateinit var sharedPreferences: SharedPreferences
    @Inject lateinit var findUserByIdUseCase: FindUserByIdUseCase
    @Inject lateinit var updateUserUseCase: UpdateUserUseCase

    private var currentUser: UserResponse? = null

    // 🔹 Configurar el selector de imágenes
    private val imagePickerLauncher = registerForActivityResult(
        ActivityResultContracts.GetContent()
    ) { uri: Uri? ->
        uri?.let {
            selectedImageUri = it
            Glide.with(this)
                .load(it)
                .override(300, 300)
                .transform(CircleCrop())
                .into(binding.imageViewProfile)
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentSettingsBinding.inflate(inflater, container, false)

        // 🔹 Cargar datos del usuario
        loadUserData()

        // 🔹 Click en la imagen para abrir la galería
        binding.imageViewProfile.setOnClickListener {
            imagePickerLauncher.launch("image/*")
        }

        // 🔹 Guardar cambios cuando se pulsa el botón
        binding.buttonSaveSettings.setOnClickListener {
            updateUserData()
        }


        return binding.root
    }

    private fun loadUserData() {
        val token = sharedPreferences.getString("auth_token", "") ?: ""
        val userId = sharedPreferences.getInt("user_id", -1)

        if (token.isNotEmpty() && userId > 0) {
            lifecycleScope.launch {
                try {
                    val user = findUserByIdUseCase(token, userId)
                    if (user != null) {
                        currentUser = user
                        binding.editTextEmail.text = user.email
                        setUserInfo(user)
                    } else {
                        showToast("No se pudo cargar la información del usuario")
                    }
                } catch (e: Exception) {
                    showToast("Error al obtener usuario: ${e.message}")
                }
            }
        }

    }

    private fun setUserInfo(user: UserResponse) {
        binding.editTextUsername.setText(user.username)

        if (!user.profilePhoto.isNullOrEmpty()) {
            Glide.with(this)
                .load(user.profilePhoto)
                .override(300, 300)
                .transform(CircleCrop())
                .into(binding.imageViewProfile)
        }
    }

    private fun updateUserData() {
        val token = sharedPreferences.getString("auth_token", "") ?: ""
        val userId = sharedPreferences.getInt("user_id", -1)

        if (token.isEmpty() || userId <= 0 || currentUser == null) {
            showToast("No se pudo actualizar el usuario.")
            return
        }

        val newUsername = binding.editTextUsername.text.toString().trim()
        val newProfilePhoto = selectedImageUri?.let { uriToBase64(it) } ?: currentUser?.profilePhoto ?: ""

        if (newUsername.isEmpty()) {
            showToast("El nombre de usuario no puede estar vacío")
            return
        }

        val updatedUser = UserAllAttributesDto(
            rol = currentUser!!.rol,
            id = userId,
            username = newUsername,
            email = currentUser!!.email,  // No permitimos editar el email
            password = currentUser!!.password,  // No permitimos editar la contraseña aquí
            profilePhoto = newProfilePhoto
        )

        lifecycleScope.launch {
            try {
                val response = updateUserUseCase(token, updatedUser)
                if (response != null) {
                    showToast("Usuario actualizado correctamente")

                    // 🔹 Actualizar los valores guardados en SharedPreferences
                    sharedPreferences.edit()
                        .putString("username", newUsername)
                        .putString("uri", newProfilePhoto)
                        .apply()

                    // 🔹 Actualizar UI del NavView en MainActivity
                    updateMainActivityUI(newUsername, newProfilePhoto)
                } else {
                    showToast("Error al actualizar el usuario")
                }
            } catch (e: Exception) {
                showToast("Error al actualizar: ${e.message}")
            }
        }
    }

    private fun updateMainActivityUI(username: String, profilePhoto: String) {
        val mainActivity = requireActivity() as MainActivity
        val headerView = mainActivity.binding.myNavView.getHeaderView(0)

        headerView.findViewById<TextView>(R.id.txt_name).text = username

        Glide.with(this)
            .load(profilePhoto)
            .override(300, 300)
            .transform(CircleCrop())
            .into(headerView.findViewById(R.id.image_perfil))
    }

    private fun showToast(message: String) {
        Toast.makeText(requireContext(), message, Toast.LENGTH_SHORT).show()
    }
    // 🔹 Método que convierte imágenes a base 64
    private fun uriToBase64(uri: Uri): String? {
        return try {
            val contentResolver: ContentResolver = requireContext().contentResolver
            val inputStream: InputStream? = contentResolver.openInputStream(uri)

            // 🔹 Leer bytes del archivo
            val byteArray = inputStream?.use { it.readBytes() } ?: return null

            // 🔹 Codificar en Base64 sin saltos de línea
            val base64Encoded = Base64.encodeToString(byteArray, Base64.NO_WRAP)

            // 🔹 Detectar tipo MIME correctamente
            val mimeType = contentResolver.getType(uri) ?: "image/jpeg"
            val prefix = when {
                mimeType.contains("jpeg", ignoreCase = true) -> "data:image/jpeg;base64,"
                mimeType.contains("png", ignoreCase = true) -> "data:image/png;base64,"
                mimeType.contains("gif", ignoreCase = true) -> "data:image/gif;base64,"
                else -> "data:image/jpeg;base64," // Default
            }

            "$prefix$base64Encoded"

        } catch (e: Exception) {
            e.printStackTrace()
            showToast("Error al convertir imagen a Base64")
            null
        }
    }
}
