package com.example.f1hub.ui.view.fragment.content

import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.fragment.app.Fragment
import androidx.fragment.app.activityViewModels
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import com.example.f1hub.R
import com.example.f1hub.databinding.FragmentHomeBinding
import com.example.f1hub.ui.adapter.AdapterItem
import com.example.f1hub.ui.view.MainActivity
import com.example.f1hub.ui.view.fragment.dialogues.DialogAdd
import com.example.f1hub.ui.view.fragment.dialogues.DialogConfirmacion
import com.example.f1hub.ui.view.fragment.dialogues.DialogEdit
import com.example.f1hub.ui.view.login.Login
import com.example.f1hub.ui.viewmodel.ItemViewModel
import dagger.hilt.android.AndroidEntryPoint
import javax.inject.Inject

@AndroidEntryPoint
class HomeFragment : Fragment() {

    private lateinit var binding: FragmentHomeBinding

    // ViewModel con inyección de Hilt
    private val viewModel: ItemViewModel by activityViewModels()

    @Inject
    lateinit var adapterFactory: AdapterItem.Factory

    @Inject
    lateinit var sharedPreferences: SharedPreferences

    private lateinit var adapter: AdapterItem

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        binding = FragmentHomeBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // Inicializar Adapter correctamente con AssistedInject
        adapter = adapterFactory.create(
            mutableListOf(),
            { id -> showDeleteConfirmationDialog(id) }, // Eliminar ítem
            { id -> showEditDialog(id) }, // Editar ítem
            { id -> onItemClick(id) } // Click en imagen
        )

        // Configurar RecyclerView
        binding.myRecyclerView.layoutManager = LinearLayoutManager(requireContext())
        binding.myRecyclerView.adapter = adapter

        // Observar los cambios en la lista de items
        viewModel.itemsLiveData.observe(viewLifecycleOwner, Observer { items ->
            adapter.submitList(items.toMutableList()) // Convertimos a MutableList para evitar errores
            if (viewModel.itemInserted) {
                binding.myRecyclerView.smoothScrollToPosition(items.size - 1)
                viewModel.itemInserted = false
            }
        })

        // Observar el estado del ProgressBar
        viewModel.progressBar.observe(viewLifecycleOwner, Observer { isLoading ->
            binding.progressBar.visibility = if (isLoading) View.VISIBLE else View.GONE
        })

        // Cargar items al inicio
        viewModel.getItems()

        //Observar la validez del token
        viewModel.navigateToLogin.observe(viewLifecycleOwner) { shouldNavigate ->
            if (shouldNavigate) {
                startActivity(Intent(requireActivity() as MainActivity, Login::class.java))
            }
        }

        binding.add.setOnClickListener { showAddDialog() }
    }

    private fun showAddDialog() {
        val dialog = DialogAdd ({ newItem, onResponse ->
            viewModel.addItem(newItem) { responseItem ->
                if (responseItem != null) {
                    showToast("Ítem agregado: ${responseItem.title}")
                } else {
                    showToast("Error al agregar el ítem")
                }
                onResponse(responseItem) // Se actualiza la lista si la API responde correctamente
            }
        },sharedPreferences)
        dialog.show(parentFragmentManager, "add_item")
    }

    private fun showEditDialog(id: Int) {
        viewModel.itemsLiveData.value?.find { it.id.toInt() == id }?.let { currentItem ->
            val dialog = DialogEdit(currentItem, { updatedItem, onResponse ->
                viewModel.updateItem(updatedItem) { responseItem ->
                    if (responseItem != null) {
                        showToast("Ítem actualizado: ${responseItem.title}")
                    } else {
                        showToast("Error al actualizar el ítem")
                    }
                    onResponse(responseItem) // Se actualiza la lista si la API responde correctamente
                }
            },sharedPreferences)
            dialog.show(parentFragmentManager, "edit_item")
        }
    }

    private fun showDeleteConfirmationDialog(id: Int) {
        val dialog = DialogConfirmacion(object : DialogConfirmacion.OnDeleteConfirmedListener {
            override fun onDeleteConfirmed(onResult: (Boolean) -> Unit) {
                viewModel.deleteItem(id) { isDeleted ->
                    if (isDeleted) {
                        showToast("Ítem eliminado correctamente")
                    } else {
                        showToast("No se pudo eliminar el ítem")
                    }
                    onResult(isDeleted) // Se actualiza la UI si la API responde correctamente
                }
            }
        })
        dialog.show(parentFragmentManager, "delete_confirmation")
    }

    private fun onItemClick(id: Int) {
        val bundle = Bundle().apply {
            putInt("id", id)
            putString("tipoItem", "item")
        }
        val navController = (requireActivity() as MainActivity).navController
        navController.navigate(R.id.infoItemsFragment, bundle)
    }

    private fun showToast(message: String) {
        Toast.makeText(requireContext(), message, Toast.LENGTH_SHORT).show()
    }
}
