package com.example.f1hub.ui.view.fragment.dialogues

import android.app.AlertDialog
import android.app.Dialog
import android.content.ContentResolver
import android.content.SharedPreferences
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.net.Uri
import android.os.Bundle
import android.util.Base64
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.fragment.app.DialogFragment
import com.bumptech.glide.Glide
import com.bumptech.glide.load.resource.bitmap.CircleCrop
import com.example.f1hub.R
import com.example.f1hub.data.dto.out.items.ItemsNoIdDto
import com.example.f1hub.data.dto.`in`.items.ResponseItem
import com.example.f1hub.databinding.FragmentDialogAddBinding
import com.example.f1hub.di.AppModule
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import java.io.InputStream


class DialogAdd(
    private val addItemDialog: (ItemsNoIdDto, (ResponseItem?) -> Unit) -> Unit,
    private val sharedPreferences: SharedPreferences
) : DialogFragment() {

    private var _binding: FragmentDialogAddBinding? = null
    private val binding get() = _binding!!

    private var selectedImageUri: Uri? = null // URI de la imagen seleccionada

    private val imagePickerLauncher = registerForActivityResult(
        ActivityResultContracts.GetContent()
    ) { uri: Uri? ->
        uri?.let {
            selectedImageUri = it
            Glide.with(this)
                .load(it)
                .override(300, 300)
                .transform(CircleCrop())
                .into(binding.imagePreview)
        }
    }

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val builder = AlertDialog.Builder(requireActivity())
        _binding = FragmentDialogAddBinding.inflate(layoutInflater)

        // Click en la imagen para abrir la galería
        binding.imagePreview.setOnClickListener {
            imagePickerLauncher.launch("image/*")
        }

        builder.setView(binding.root)
            .setPositiveButton("Aceptar") { _, _ ->

                val addItem = recoverDataLayout()
                if (addItem.title.isEmpty() || addItem.country.isEmpty() ||
                    addItem.description.isEmpty() || addItem.type.isEmpty()
                ) {
                    showToast("Algún campo está vacío")

                }

                addItemDialog(addItem) { response ->
                    response?.let { showSuccessMessage(it) }
                }
            }
            .setNegativeButton("Cancelar") { dialog, _ -> dialog.dismiss() }

        return builder.create()
    }

    private fun recoverDataLayout(): ItemsNoIdDto {
        val type = resources.getStringArray(R.array.type)
            .getOrElse(binding.type.selectedItemPosition) { "" }

        val base64Image = selectedImageUri?.let { uriToBase64(it) }

        return ItemsNoIdDto(
            country = binding.countryInput.text.toString(),
            title = binding.title.text.toString(),
            type = type,
            description = binding.editTextArea.text.toString(),
            image = base64Image,
            userId = AppModule.provideUserId(sharedPreferences)
        )
    }

    // 🔹 Método que convierte imágenes a base 64
    private fun uriToBase64(uri: Uri): String? {
        return try {
            val contentResolver: ContentResolver = requireContext().contentResolver
            val inputStream: InputStream? = contentResolver.openInputStream(uri)

            // 🔹 Leer bytes del archivo
            val byteArray = inputStream?.use { it.readBytes() } ?: return null

            // 🔹 Codificar en Base64 sin saltos de línea
            val base64Encoded = Base64.encodeToString(byteArray, Base64.NO_WRAP)

            // 🔹 Detectar tipo MIME correctamente
            val mimeType = contentResolver.getType(uri) ?: "image/jpeg"
            val prefix = when {
                mimeType.contains("jpeg", ignoreCase = true) -> "data:image/jpeg;base64,"
                mimeType.contains("png", ignoreCase = true) -> "data:image/png;base64,"
                mimeType.contains("gif", ignoreCase = true) -> "data:image/gif;base64,"
                else -> "data:image/jpeg;base64," // Default
            }

            "$prefix$base64Encoded"

        } catch (e: Exception) {
            e.printStackTrace()
            showToast("Error al convertir imagen a Base64")
            null
        }
    }



    private fun showSuccessMessage(responseItem: ResponseItem) {
        showToast("Ítem agregado: ${responseItem.title}")
    }

    private fun showToast(message: String) {
        CoroutineScope(Dispatchers.Main).launch {
            context?.let {
                Toast.makeText(it, message, Toast.LENGTH_LONG).show()
            }
        }
    }

}
