package com.example.f1hub.ui.view.fragment.dialogues

import android.app.AlertDialog
import android.app.Dialog
import android.content.ContentResolver
import android.content.SharedPreferences
import android.net.Uri
import android.os.Bundle
import android.util.Base64
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.fragment.app.DialogFragment
import com.bumptech.glide.Glide
import com.bumptech.glide.load.resource.bitmap.CircleCrop
import com.example.f1hub.R
import com.example.f1hub.data.dto.out.items.ItemsAllAttributesDto
import com.example.f1hub.data.dto.`in`.items.ResponseItem
import com.example.f1hub.databinding.FragmentDialogEditBinding
import com.example.f1hub.di.AppModule
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import java.io.InputStream

class DialogEdit(
    private val itemToUpdate: ResponseItem,
    private val updateItemDialog: (ItemsAllAttributesDto, (ResponseItem?) -> Unit) -> Unit,
    private val sharedPreferences: SharedPreferences
) : DialogFragment() {

    private var _binding: FragmentDialogEditBinding? = null
    private val binding get() = _binding!!

    private var selectedImageUri: Uri? = null

    // 🚀 Lanzador para seleccionar imagen desde galería
    private val imagePickerLauncher = registerForActivityResult(
        ActivityResultContracts.GetContent()
    ) { uri: Uri? ->
        uri?.let {
            selectedImageUri = it
            Glide.with(this)
                .load(it)
                .override(300, 300)
                .transform(CircleCrop())
                .into(binding.imagePreview)
        }
    }

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val builder = AlertDialog.Builder(requireActivity())
        _binding = FragmentDialogEditBinding.inflate(layoutInflater)

        // 🔹 Rellenar los datos actuales
        setValuesIntoDialog()

        // 🔹 Click en la imagen para cambiarla
        binding.imagePreview.setOnClickListener {
            imagePickerLauncher.launch("image/*")
        }

        builder.setView(binding.root)
            .setPositiveButton("Guardar") { _, _ ->

                val updatedItem = recoverDataLayout()
                if (updatedItem.title.isEmpty() || updatedItem.country.isEmpty() ||
                    updatedItem.description.isEmpty() || updatedItem.type.isEmpty()
                ) {
                    showToast("Algún campo está vacío")
                    return@setPositiveButton
                }

                // 🔹 Enviar el objeto actualizado y manejar la respuesta
                updateItemDialog(updatedItem) { response ->
                    response?.let { showSuccessMessage(it) }
                }
            }
            .setNegativeButton("Cancelar") { dialog, _ -> dialog.dismiss() }

        return builder.create()
    }

    private fun setValuesIntoDialog() {
        val arrayTypes = resources.getStringArray(R.array.type)

        binding.title.setText(itemToUpdate.title)
        binding.editTextArea.setText(itemToUpdate.description)
        binding.countryInput.setText(itemToUpdate.country)

        val typeIndex = arrayTypes.indexOf(itemToUpdate.type)
        binding.type.setSelection(if (typeIndex >= 0) typeIndex else 0)

        // 🔹 Cargar imagen desde URL o mantener la predeterminada
        Glide.with(this)
            .load(itemToUpdate.image)
            .override(300, 300)
            .transform(CircleCrop())
            .into(binding.imagePreview)
    }

    private fun recoverDataLayout(): ItemsAllAttributesDto {
        val type = resources.getStringArray(R.array.type)
            .getOrElse(binding.type.selectedItemPosition) { "" }

        val base64Image = selectedImageUri?.let { uriToBase64(it) } ?: null

        return ItemsAllAttributesDto(
            id = itemToUpdate.id.toIntOrNull() ?: 0,
            title = binding.title.text.toString(),
            description = binding.editTextArea.text.toString(),
            country = binding.countryInput.text.toString(),
            type = type,
            image = base64Image,
            userId = AppModule.provideUserId(sharedPreferences)
        )
    }

    private fun uriToBase64(uri: Uri): String? {
        return try {
            val contentResolver: ContentResolver = requireContext().contentResolver
            val inputStream: InputStream? = contentResolver.openInputStream(uri)

            val byteArray = inputStream?.use { it.readBytes() } ?: return null
            val base64Encoded = Base64.encodeToString(byteArray, Base64.NO_WRAP)

            val mimeType = contentResolver.getType(uri) ?: "image/jpeg"
            val prefix = when {
                mimeType.contains("jpeg", ignoreCase = true) -> "data:image/jpeg;base64,"
                mimeType.contains("png", ignoreCase = true) -> "data:image/png;base64,"
                mimeType.contains("gif", ignoreCase = true) -> "data:image/gif;base64,"
                else -> "data:image/jpeg;base64,"
            }

            "$prefix$base64Encoded"

        } catch (e: Exception) {
            e.printStackTrace()
            showToast("Error al convertir imagen a Base64")
            null
        }
    }

    private fun showSuccessMessage(responseItem: ResponseItem) {
        showToast("Ítem actualizado: ${responseItem.title}")
    }

    private fun showToast(message: String) {
        CoroutineScope(Dispatchers.Main).launch {
            context?.let {
                Toast.makeText(it, message, Toast.LENGTH_LONG).show()
            }

        }
    }
}
