package com.example.f1hub.ui.view.login

import android.content.Intent
import android.os.Bundle
import android.view.View
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.lifecycle.lifecycleScope
import com.example.f1hub.databinding.ActivityLoginBinding
import com.example.f1hub.di.AppModule
import com.example.f1hub.domain.usecase.users.LoginUseCase
import com.example.f1hub.domain.usecase.users.ValidateTokenUseCase
import com.example.f1hub.ui.view.MainActivity
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.launch
import javax.inject.Inject

@AndroidEntryPoint
class Login : AppCompatActivity() {

    private lateinit var binding: ActivityLoginBinding

    @Inject lateinit var loginUseCase: LoginUseCase
    @Inject lateinit var validateTokenUseCase: ValidateTokenUseCase
    @Inject lateinit var sharedPreferences: android.content.SharedPreferences

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityLoginBinding.inflate(layoutInflater)
        setContentView(binding.root)

        binding.progressBar.visibility = View.GONE

        checkExistingToken() // 🔹 Verificar si ya hay un token válido
        setupListeners()
    }

    private fun checkExistingToken() {
        val token = AppModule.provideToken(sharedPreferences)

        if (token.isNotEmpty()) {
            lifecycleScope.launch {
                binding.progressBar.visibility = View.VISIBLE

                val isValid = validateTokenUseCase(token)

                if (isValid) {
                    navigateToMain()
                } else {
                    // 🔹 Token inválido, eliminarlo y seguir en login
                    removeToken()
                    showToast("Sesión expirada. Inicia sesión de nuevo.")
                }

                binding.progressBar.visibility = View.GONE
            }
        }
    }

    private fun setupListeners() {
        binding.loginButton.setOnClickListener {
            val email = binding.username.text.toString().trim()
            val password = binding.password.text.toString().trim()

            if (email.isEmpty() || password.isEmpty()) {
                showToast("Tienes algún campo vacío")
                return@setOnClickListener
            }

            binding.progressBar.visibility = View.VISIBLE
            loginUser(email, password)
        }

        binding.registerButton.setOnClickListener {
            binding.progressBar.visibility = View.VISIBLE
            navigateToRegister()
        }
    }

    private fun loginUser(email: String, password: String) {
        lifecycleScope.launch {
            binding.progressBar.visibility = View.VISIBLE

            try {
                val userLoginDto = com.example.f1hub.data.dto.out.users.UserLoginDto(email, password)
                val response = loginUseCase(userLoginDto)

                if (response != null && response.id.toInt() > 0) {
                    // 🔹 Guardar token en SharedPreferences
                    saveToken(response.token,response.id.toInt())
                    sharedPreferences.edit()
                        .putInt("user_id", response.id.toInt())
                        .apply()

                    showToast("Inicio de sesión exitoso")
                    navigateToMain()
                } else {
                    showToast("Credenciales incorrectas")
                }
            } catch (e: Exception) {
                showToast("Error al iniciar sesión: ${e.message}")
            } finally {
                binding.progressBar.visibility = View.GONE
            }
        }
    }

    private fun navigateToMain() {
        startActivity(Intent(this, MainActivity::class.java))
        finish()
    }
    private fun navigateToRegister() {
        startActivity(Intent(this, Register::class.java))
        finish()
    }

    private fun saveToken(token: String, userId: Int) {
        sharedPreferences.edit()
            .putString("auth_token", "Bearer $token")
            .putInt("user_id", userId)
            .apply()
    }

    private fun removeToken() {
        sharedPreferences.edit()
            .remove("auth_token")
            .remove("user_id")
            .apply()
    }

    private fun showToast(message: String) {
        Toast.makeText(this, message, Toast.LENGTH_LONG).show()
    }
}
