package com.example.f1hub.ui.viewmodel

import android.content.SharedPreferences
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.f1hub.data.dto.out.items.ItemsAllAttributesDto
import com.example.f1hub.data.dto.out.items.ItemsNoIdDto
import com.example.f1hub.data.dto.`in`.items.ResponseItem
import com.example.f1hub.di.AppModule
import com.example.f1hub.domain.usecase.items.AddItemUseCase
import com.example.f1hub.domain.usecase.items.DeleteItemUseCase
import com.example.f1hub.domain.usecase.items.EditItemUseCase
import com.example.f1hub.domain.usecase.items.GetAllItemsByUser
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class ItemViewModel @Inject constructor(
    private val sharedPreferences: SharedPreferences,
    private val getAllItemsUseCase: GetAllItemsByUser,
    private val updateItemsUseCase: EditItemUseCase,
    private val addItemsUseCase: AddItemUseCase,
    private val deleteItemsUseCase: DeleteItemUseCase

) : ViewModel() {

    private val _itemsLiveData = MutableLiveData<List<ResponseItem>>()
    val itemsLiveData: LiveData<List<ResponseItem>> = _itemsLiveData

    private val _progressBar = MutableLiveData<Boolean>()
    val progressBar: LiveData<Boolean> = _progressBar

    private val _navigateToLogin = MutableLiveData<Boolean>()
    val navigateToLogin: LiveData<Boolean> = _navigateToLogin

    var itemInserted = false

    fun getItems() {
        viewModelScope.launch {
            _progressBar.value = true
            try {
                val items = getAllItemsUseCase(AppModule.provideToken(sharedPreferences),AppModule.provideUserId(sharedPreferences))
                _itemsLiveData.value = items
            } catch (e: Exception) {
                if (e.message?.contains("401") == true) { // Si hay 401, borrar token y redirige al login
                    handleUnauthorized()
                }
            }
            _progressBar.value = false
        }
    }

    fun addItem(newItem: ItemsNoIdDto, onResult: (ResponseItem?) -> Unit) {
        viewModelScope.launch {
            _progressBar.value = true
            try {
                val responseItem = addItemsUseCase(AppModule.provideToken(sharedPreferences), newItem)
                itemInserted = responseItem != null
                onResult(responseItem)
                getItems()
            } catch (e: Exception) {
                if (e.message?.contains("401") == true) { // Si hay 401, borrar token y redirige al login
                    handleUnauthorized()
                }
            }
            _progressBar.value = false
        }
    }

    fun updateItem(updatedItem: ItemsAllAttributesDto, onResult: (ResponseItem?) -> Unit) {
        viewModelScope.launch {
            _progressBar.value = true
            try {
                val responseItem = updateItemsUseCase(AppModule.provideToken(sharedPreferences), updatedItem)
                itemInserted = false
                onResult(responseItem)
                getItems()
            } catch (e: Exception) {
                if (e.message?.contains("401") == true) { // Si hay 401, borrar token y redirige al login
                    handleUnauthorized()
                }
            }
            _progressBar.value = false
        }
    }

    fun deleteItem(id: Int, onResult: (Boolean) -> Unit) {
        viewModelScope.launch {
            try {
                _progressBar.value = true
                val wasDeleted = deleteItemsUseCase(AppModule.provideToken(sharedPreferences), id)
                _progressBar.value = false
                onResult(wasDeleted)
                getItems()
            } catch (e: Exception) {
                if (e.message?.contains("401") == true) { // Si hay 401, borrar token y redirige al login
                    handleUnauthorized()
                }
            }

        }
    }

    private fun handleUnauthorized() {
        sharedPreferences.edit()
            .remove("auth_token")
            .remove("user_id")
            .apply()
        _navigateToLogin.value = true
    }
}
